#ifndef __SGLC_H__
#define __SGLC_H__

#define SGLC_API __declspec(dllimport) int _stdcall

//return code definition
#define MOT_COMM_OK                 0       //went through and honored
#define MOT_COMM_ERROR_CONNECT      -1      //can't connect realtime task
#define MOT_COMM_ERROR_TIMEOUT      -2      //send command timeout
#define MOT_COMM_ERROR_COMMAND      -3      //controller can't run command now
#define MOT_COMM_READ_STATUS_TIMEOUT -4     //read status timeout
#define MOT_COMM_INVALID_ARGUMENT   -5      //invalid argument
#define MOT_ERROR_ALREADY_EXISTS        -6  //device already open
#define MOT_ERROR_INSUFFICIENT_RESOURCE -7  //cannot create mutex

//command accepted by built-in XY2-100 controller
#define XY2_CMD_POS_X(d) ((0x4u << 28) | ((d) & 0xfffffff))     //store X pos
#define XY2_CMD_POS_Y(d) ((0x5u << 28) | ((d) & 0xfffffff))     //store Y pos and then send XYZ to shift register
#define XY2_CMD_POS_Z(d) ((0x6u << 28) | ((d) & 0xfffffff))     //store Z pos
#define XY2_CMD_DELAY(d) ((0x7u << 28) | ((d) & 0xfffffff))     //delay(0~1048575), unit in 10-microsecond,
#define XY2_CMD_DOUT(d)    ((0x8u << 28) | ((d) & 0xfffffff))   //set dout port(DO0~DO23)
#define XY2_CMD_AOUT(d)    ((0x9u << 28) | ((d) & 0xfffffff))   //set analog output(reserved for future)
#define XY2_CMD_DOUT_BIT(d,n) ((0xau << 28) | (((d) & 0x1) << 24) | (n & 0x1f))
                                                                //set a dout bit[n] with d
#define XY2_CMD_PWMW(d)    ((0xbu << 28) | ((d) & 0xfffffff))   //store PWM width(0~65535) in 40-nanosecond
#define XY2_CMD_PWMP(d)    ((0xcu << 28) | ((d) & 0xfffffff))   //store PWM period(0~65535) in 40-nanosecond and then set PWM controller


//bit definition of digital output
//[16:23] - TTL Dout[0:7]
#define LC_DOUT_NUM_ESTOP   (12)
#define LC_DOUT_NUM_MO      (11)
#define LC_DOUT_NUM_AP      (10)
#define LC_DOUT_NUM_REDPT   (9)
#define LC_DOUT_NUM_PLATCH  (8)
#define LC_DOUT_NUM_P7      (7)
#define LC_DOUT_NUM_P6      (6)
#define LC_DOUT_NUM_P5      (5)
#define LC_DOUT_NUM_P4      (4)
#define LC_DOUT_NUM_P3      (3)
#define LC_DOUT_NUM_P2      (2)
#define LC_DOUT_NUM_P1      (1)
#define LC_DOUT_NUM_P0      (0)

#ifdef __cplusplus
extern "C" {
#endif
SGLC_API LC_Open(void);
SGLC_API LC_Close(void);

SGLC_API LC_SubmitBuffer(int dev, unsigned int* buffer, unsigned int count, unsigned int* act);
SGLC_API LC_GetStatus(int dev, unsigned int* buffer, unsigned int count);
SGLC_API LC_Abort(int dev);
SGLC_API LC_SetDout(int dev, int index, int value);

SGLC_API LC_GetDevInfo(int dev, unsigned int* buffer, unsigned int count);
SGLC_API LC_Ind(int dev, unsigned int offset, unsigned int* value);
SGLC_API LC_Outd(int dev, unsigned int offset, unsigned int value);
#ifdef __cplusplus
}
#endif

#endif
