﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SGLC;

namespace Demo1
{
    public partial class FormManual : Form
    {
        public FormManual()
        {
            InitializeComponent();
        }
        /// <summary>
        /// 读硬件寄存器
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnReadReg_Click(object sender, EventArgs e)
        {
            uint offset, d = 0;
            offset = Convert.ToUInt32(txtRegOffset.Text, 16);
            //d = Convert.ToUInt32(txtRegWriteValue.Text);
            pcie6603.LC_Ind(0, offset, ref d);
            txtRegReadValue.Text = d.ToString("x8");

            NcConfig.halOffset[0] = offset;
        }
        /// <summary>
        /// 写硬件寄存器
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnWriteReg_Click(object sender, EventArgs e)
        {
            uint offset, d;
            offset = Convert.ToUInt32(txtRegOffset.Text, 16);
            d = Convert.ToUInt32(txtRegWriteValue.Text, 16);
            pcie6603.LC_Outd(0, offset, d);

            NcConfig.halOffset[0] = offset;
        }

        /// <summary>
        /// 设置单次PWM（输出定时）
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOutputSinglePWM_Click(object sender, EventArgs e)
        {
            if (NcConfig.fpgaIs0102[0] == false)
            {
                MessageBox.Show("此版本的FPGA不支持该特性！", "错误");
                return;
            }

            uint d = 0;
            d = Convert.ToUInt32(txtPulseWidth.Text);

            NcConfig.pulseWidth[0] = d;

            d = d * 25; //conert to ns

            //直接写寄存器方式
            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_PULSE_WIDTH(d));

            /*
            //使用缓冲区方式
            uint actCnt = 0;
            uint[] cmd = new uint[1024 * 1024];
            cmd[0] = d;
            cmd[1] = pcie6603.XY2_CMD_PULSE_WIDTH(d);
            pcie6603.LC_SubmitBuffer(0, cmd, 2, ref actCnt);
            */
        }
        /// <summary>
        /// 设置数字输出，DO[0]~DO[23]通过直接写硬件指令队列方式实现；DO[24]~DO[31]通过API实现
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDoutON_Click(object sender, EventArgs e)
        {
            int index = 0;
            uint cmd = 0;
            uint[] stat = new uint[32];
            int rc = 0;

            index = cbDoutIndex.SelectedIndex;
            if (index < 0)
                return;
            //TTL output
            if (index < 24L)
            {
                rc = pcie6603.LC_GetStatus(0, stat, 16);
                if (rc == 0)
                {
                    if (stat[9] > 0)
                    {
                        MessageBox.Show("控制器处于忙状态，不可直接设置指令队列！");
                        return;
                    }
                    else
                    {
                        cmd = pcie6603.XY2_CMD_DOUT_BIT(1, Convert.ToUInt32(index));
                        pcie6603.LC_Outd(0, 0x1090, cmd);
                    }
                }
                else
                {
                    MessageBox.Show("查询控制器状态失败！");
                }
            }
            //OC output
            else
            {
                pcie6603.LC_SetDout(0, index, 1);
            }
        }
        /// <summary>
        /// 设置数字输出，DO[0]~DO[23]通过直接写硬件指令队列方式实现；DO[24]~DO[31]通过API实现
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDoutOFF_Click(object sender, EventArgs e)
        {
            int index = 0;
            uint cmd = 0;
            uint[] stat = new uint[32];
            int rc = 0;

            index = cbDoutIndex.SelectedIndex;
            if (index < 0)
                return;
            //TTL output
            if (index < 24L)
            {
                rc = pcie6603.LC_GetStatus(0, stat, 16);
                if (rc == 0)
                {
                    if (stat[9] > 0)
                    {
                        MessageBox.Show("控制器处于忙状态，不可直接设置指令队列！");
                        return;
                    }
                    else
                    {
                        cmd = pcie6603.XY2_CMD_DOUT_BIT(0, Convert.ToUInt32(index));
                        pcie6603.LC_Outd(0, 0x1090, cmd);
                    }
                }
                else
                {
                    MessageBox.Show("查询控制器状态失败！");
                }
            }
            //OC output
            else
            {
                pcie6603.LC_SetDout(0, index, 0);
            }
        }
        /// <summary>
        /// 设置连续PWM输出参数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSetContPWM_Click(object sender, EventArgs e)
        {
            uint p, w;
            p = Convert.ToUInt32(txtPWMPeriod.Text);
            w = Convert.ToUInt32(txtPWMWidth.Text);

            if ((p > 65535u * 40u) || (w > 65535u * 40u))
            {
                MessageBox.Show("参数超出范围(0~2621400", "错误");
                return ;
            }

            NcConfig.pwmWidth[0] = w;
            NcConfig.pwmPeriod[0] = p;

            p = p / 40;
            w = w / 40;
            //直接写寄存器方式
            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_PWMW(w));
            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_PWMP(p));

            /*
            //使用缓冲区方式
            uint actCnt = 0;
            uint[] cmd = new uint[1024 * 1024];
            cmd[0] = pcie6603.XY2_CMD_PWMW(w);
            cmd[1] = pcie6603.XY2_CMD_PWMP(p);
            pcie6603.LC_SubmitBuffer(0, cmd, 2, ref actCnt);
            */
        }
        /// <summary>
        /// 设置振镜位置
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSetMirrorPos_Click(object sender, EventArgs e)
        {
            ushort[] pos = new ushort[3];

            pos[0] = Convert.ToUInt16(txtMirrorPos0.Text);
            pos[1] = Convert.ToUInt16(txtMirrorPos1.Text);
            pos[2] = Convert.ToUInt16(txtMirrorPos2.Text);

            NcConfig.mirrorPos0[0] = pos[0];
            NcConfig.mirrorPos1[0] = pos[1];
            NcConfig.mirrorPos2[0] = pos[2];

            //直接写寄存器方式
            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_POS_Z(pos[2]));

            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_POS_X(pos[0]));
            pcie6603.LC_Outd(0, 0x1090, pcie6603.XY2_CMD_POS_Y(pos[1]));  //load mirror pos to xy2

            /*
            //使用缓冲区方式
            uint actCnt = 0;
            uint[] cmd = new uint[1024 * 1024];
            cmd[0] = pcie6603.XY2_CMD_POS_Z(d[2]);
            cmd[1] = pcie6603.XY2_CMD_POS_X(d[0]);
            cmd[2] = pcie6603.XY2_CMD_POS_Y(d[1]);
            pcie6603.LC_SubmitBuffer(0, cmd, 3, ref actCnt);
            */
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (chkRegAutoRead.Checked)
            {
                uint offset, d = 0;
                offset = Convert.ToUInt32(txtRegOffset.Text, 16);
                pcie6603.LC_Ind(0, offset, ref d);
                txtRegReadValue.Text = d.ToString("x8");
            }
        }

        private void FormManual_Load(object sender, EventArgs e)
        {
            cbDoutIndex.SelectedIndex = 24;
            timer1.Enabled = true;

            txtPWMPeriod.Text = NcConfig.pwmPeriod[0].ToString();
            txtPWMWidth.Text = NcConfig.pwmWidth[0].ToString();

            txtPulseWidth.Text = NcConfig.pulseWidth[0].ToString();

            txtMirrorPos0.Text = NcConfig.mirrorPos0[0].ToString();
            txtMirrorPos1.Text = NcConfig.mirrorPos1[0].ToString();
            txtMirrorPos2.Text = NcConfig.mirrorPos2[0].ToString();

            txtRegOffset.Text = NcConfig.halOffset[0].ToString("x8");
        }
    }
}
