﻿using System;
using System.Runtime.InteropServices;
namespace SGLC
{
    public class pcie6603
    {
        /// return code definition
        public const int MOT_COMM_OK = 0;
        public const int MOT_COMM_ERROR_CONNECT = -1;
        public const int MOT_COMM_ERROR_TIMEOUT = -2;
        public const int MOT_COMM_ERROR_COMMAND = -3;
        public const int MOT_COMM_READ_STATUS_TIMEOUT = -4;
        public const int MOT_COMM_INVALID_ARGUMENT = -5;
        public const int MOT_ERROR_ALREADY_EXISTS = -6;
        public const int MOT_ERROR_INSUFFICIENT_RESOURCE = -7;

        //bit definition of digital output
        //[16:23] - TTL Dout[0:7]
        public const int LC_DOUT_NUM_ESTOP = 12;
        public const int LC_DOUT_NUM_MO = 11;
        public const int LC_DOUT_NUM_AP = 10;
        public const int LC_DOUT_NUM_REDPT = 9;
        public const int LC_DOUT_NUM_PLATCH = 8;
        public const int LC_DOUT_NUM_P7 = 7;
        public const int LC_DOUT_NUM_P6 = 6;
        public const int LC_DOUT_NUM_P5 = 5;
        public const int LC_DOUT_NUM_P4 = 4;
        public const int LC_DOUT_NUM_P3 = 3;
        public const int LC_DOUT_NUM_P2 = 2;
        public const int LC_DOUT_NUM_P1 = 1;
        public const int LC_DOUT_NUM_P0 = 0;

        [DllImport("SGLC.dll")]
        public static extern int LC_Open();
        [DllImport("SGLC.dll")]
        public static extern int LC_Close();
        [DllImport("SGLC.dll")]
        public static extern int LC_SubmitBuffer(int dev, uint[] buffer, uint count, ref uint act);
        [DllImport("SGLC.dll")]
        public static extern int LC_GetStatus(int dev, uint[] buffer, uint count);
        [DllImport("SGLC.dll")]
        public static extern int LC_Abort(int dev);
        [DllImport("SGLC.dll")]
        public static extern int LC_SetDout(int dev, int index, int value);

        [DllImport("SGLC.dll")]
        public static extern int LC_GetDevInfo(int dev, uint[] buffer, uint count);
        [DllImport("SGLC.dll")]
        public static extern int LC_Ind(int dev, uint offset, ref uint value);
        [DllImport("SGLC.dll")]
        public static extern int LC_Outd(int dev, uint offset, uint value);

        /// <summary>
        /// store X pos for later use
        /// </summary>
        /// <param name="pos"></param>
        /// <returns></returns>
        public static uint XY2_CMD_POS_X(int pos)
        {
            uint d = Convert.ToUInt32(pos);
            return ((0x4u << 28) | ((d) & 0xfffffff));
        }
        /// <summary>
        /// store Y pos and then send XYZ to shift register
        /// </summary>
        /// <param name="pos"></param>
        /// <returns></returns>
        public static uint XY2_CMD_POS_Y(int pos)
        {
            uint d = Convert.ToUInt32(pos);
            return ((0x5u << 28) | ((d) & 0xfffffff));
        }
        /// <summary>
        /// store Y pos for later use
        /// </summary>
        /// <param name="pos"></param>
        /// <returns></returns>
        public static uint XY2_CMD_POS_Z(int pos)
        {
            uint d = Convert.ToUInt32(pos);
            return ((0x6u << 28) | ((d) & 0xfffffff));
        }
        /// <summary>
        /// set delay, range: 0~1048575 in 10-microsecond
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static uint XY2_CMD_DELAY(uint d)
        {
            return ((0x7u << 28) | ((d) & 0xfffffff));
        }
        /// <summary>
        /// set digital output port(bit0~bit23 corresponds to DO0~DO23 respectively)
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static uint XY2_CMD_DOUT(uint d)
        {
            return ((0x8u << 28) | ((d) & 0xfffffff));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="volt"></param>
        /// <returns></returns>
        //public static uint XY2_CMD_AOUT(double volt)
        //{
        //    uint d = Convert.ToUInt32(x);
        //    return ((0x6u << 28) | ((d) & 0xfffffff));
        //}

        /// <summary>
        /// set a dout bit(bit0~bit23)
        /// </summary>
        /// <param name="d"></param>
        /// <param name="index"></param>
        /// <returns></returns>
        public static uint XY2_CMD_DOUT_BIT(uint d, uint index)
        {
            return ((0xau << 28) | (((d) & 0x1) << 24) | (index & 0x1f));
        }
        /// <summary>
        /// store PWM width for later use, unit in 40-nanosecond
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static uint XY2_CMD_PWMW(uint d)
        {
            return ((0xbu << 28) | ((d) & 0xfffffff));
        }
        /// <summary>
        /// store PWM period and then set PWM controller(switch PWM into continuous mode), unit in 40-nanosecond
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static uint XY2_CMD_PWMP(uint d)
        {
            return ((0xcu << 28) | ((d) & 0xfffffff));
        }

        /// <summary>
        /// set width of single pulse(switch PWM into single mode), unit in 40-nanosecond
        /// </summary>
        /// <param name="d">pulse width in unit of 40-nanosecond</param>
        /// <returns></returns>
        public static uint XY2_CMD_PULSE_WIDTH(uint d)
        {
            return ((0xdu << 28) | ((d) & 0xfffffff));
        }
    }
}
